"""
demo for Midway Equalization for Video
"""

from lib import base_app, build, http
from lib.misc import app_expose
from lib.base_app import init_app
import cherrypy
from cherrypy import TimeoutError
import os.path
import time
import shutil
import subprocess
import os


class app(base_app):
    """ Midway demo """

    title = "Midway Video Equalization"
    xlink_article = 'http://www.ipol.im/'

    input_nb = 1 # number of input videos
    input_max_pixels = 50000000 # max size (in pixels) of an input image
    input_max_weight = 50 * 1024 * 1024 # max size (in bytes) of an input file
    input_dtype = '3x8i' # input image expected data type
    input_ext = '.png'   # input image expected extension (ie file format)
    is_test = True       # switch to False for deployment


    default_param = {
        'sigma': '30',
        'dithering': '0'
	}

    def __init__(self):
        """
        app setup
        """
        # setup the parent class
        base_dir = os.path.dirname(os.path.abspath(__file__))
        base_app.__init__(self, base_dir)

        # select the base_app steps to expose
        # index() is generic
        app_expose(base_app.index)
        app_expose(base_app.input_select)
        app_expose(base_app.input_upload)
        # params() is modified from the template
        app_expose(base_app.params)
        # run() and result() must be defined here


    @cherrypy.expose
    @init_app
    def wait(self, sigma="10", dithering="0"):
        """
        params handling and run redirection
        """
        # save and validate the parameters
        try:
            self.cfg['param'] = {
              'sigma' : int(sigma),
              'dithering': int(dithering)}
        except ValueError:
            return self.error(errcode='badparams',
                              errmsg="The parameters must be numeric.")

        http.refresh(self.base_url + 'run?key=%s' % self.key)
        return self.tmpl_out("wait.html")

    def build(self):
        """
        program build/update
        """
        # store common file path in variables
        archive = "midway-video_1.0.tar.gz"
        #tgz_url = self.xlink_article + archive
        tgz_url = "http://dev.ipol.im/~jsanchez/" + archive
        tgz_file = self.dl_dir + archive
        prog_file1 = "midway_video"
        prog_file2 = "midway_video.sh"
        prog_file3 = "generate_output"
        log_file = self.base_dir + "build.log"
        src_dir = self.src_dir+"midway-video_1.0/"

        # get the latest source archive
        build.download(tgz_url, tgz_file)

        # extract the archive
        build.extract(tgz_file, self.src_dir)

        # Clean the bin dir
        if os.path.isdir(self.bin_dir):
            shutil.rmtree(self.bin_dir)

        # build the program
        os.chdir(self.src_dir)
        build.run("make -C %s -j4" % (src_dir),
                  stdout=log_file)
        os.mkdir(self.bin_dir)
        shutil.copy(src_dir + "bin/" + prog_file1,
                    self.bin_dir + prog_file1)
        shutil.copy(src_dir + "bin/" + prog_file2,
                    self.bin_dir + prog_file2)
        shutil.copy(src_dir + "bin/" + prog_file3,
                    self.bin_dir + prog_file3)
        # cleanup the source dir
        shutil.rmtree(self.src_dir)
        cherrypy.log('Build completed succesfully.', context='BUILD',
                     traceback=False)
        return


    def input_select_callback(self, fnames):
        '''
        Implement the callback for the input select to
        process the non-standard input
        '''
        # get the images (exr)
        png_file = fnames[0]
        video_file = png_file.replace(".png", ".mp4")
        shutil.copy(self.input_dir + video_file,
                    self.work_dir + "input_0.mp4")

    def process_input(self):
        pass

    def clone_input(self):
        self.log("cloning input from %s" % self.key)
        # get a new key
        old_work_dir = self.work_dir
        old_cfg_meta = self.cfg['meta']

        self.new_key()
        self.init_cfg()
        # copy the input files
        shutil.copy(old_work_dir + 'input_0.mp4',
                    self.work_dir + 'input_0.mp4')
        # copy cfg
        self.cfg['meta'].update(old_cfg_meta)
        self.cfg.save()
        return


    def file_isvideo(self, video):
        command = '/usr/bin/file '+ video

        p = subprocess.Popen(command, shell=True,
                             stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        output, error = p.communicate()

        ini = output.find(":")
        output = output[ini:].upper()
        is_video = False 
        if output.find('AVI') >= 0:
            is_video = True
        if output.find('MPEG') >= 0:
            is_video = True
        if output.find('MEDIA') >= 0:
            is_video = True
        if output.find('OGG') >= 0:
            is_video = True
        if output.find('VIDEO') >= 0:
            is_video = True
        if output.find('FLV') >= 0:
            is_video = True
        return is_video


    @cherrypy.expose
    @init_app
    def input_upload(self, **kwargs):
        """
        use the uploaded input images
        """

        self.new_key()
        self.init_cfg()
        for i in range(self.input_nb):
            file_up = kwargs['file_%i' % i]
            input_file = self.work_dir + 'input_%i.original.mp4' % i
            file_save = file(input_file, 'wb')
            if '' == file_up.filename:
                # missing file
                raise cherrypy.HTTPError(400, # Bad Request
                                         "Missing input file")
            size = 0
            while True:
                data = file_up.file.read(128)
                if not data:
                    break
                size += len(data)
                if size > self.input_max_weight:
                    # file too heavy
                    raise cherrypy.HTTPError(400, # Bad Request
                                            "File too large, " +
                                            "resize or use better compression")
                file_save.write(data)
            file_save.close()

            if not self.file_isvideo(input_file):
                return self.error(errcode='badparams',
                                  errmsg='video format not recognized')

            #convert the input video to mp4 and remove audio
            p3 = self.run_proc(['avconv', '-ss', '0', '-i',
                            input_file, '-t', '10',
                            '-an', '-pix_fmt', 'yuv420p',
                            self.work_dir + 'input_%i.mp4' % i])

            self.wait_proc(p3)

        msg = self.process_input()
        self.log("input uploaded")
        self.cfg['meta']['original'] = True
        self.cfg.save()

        # jump to the params page
        return self.params(msg=msg, key=self.key)


    @cherrypy.expose
    @init_app
    def run(self):
        """
        algo execution
        """
        # read the parameters
        sigma = str(self.cfg['param']['sigma'])
        dithering = str(self.cfg['param']['dithering'])
        # run the algorithm
        try:
            run_time = time.time()
            self.run_algo(sigma, dithering, timeout=self.timeout)
            self.cfg['info']['run_time'] = time.time() - run_time
        except TimeoutError:
            return self.error(errcode='timeout')
        except RuntimeError:
            return self.error(errcode='runtime')
        http.redir_303(self.base_url + 'result?key=%s' % self.key)

        # archive
        if self.cfg['meta']['original']:
            ar = self.make_archive()
            ar.add_file("input_0.mp4", "input.mp4", info="input")
            ar.add_file("output.mp4", info="output")
            ar.add_file("acc_histogram1.mp4", info="accumulated histogram1")
            ar.add_file("histogram1.mp4", info="histogram1")
            ar.add_file("acc_histogram2.mp4", info="accumulated histogram2")
            ar.add_file("histogram2.mp4", info="histogram2")
            ar.add_info({"radius": sigma, "dithering": dithering})
            ar.save()

        return self.tmpl_out("run.html")

    def run_algo(self, sigma, dithering, timeout):
        """
        the core algo runner
        could also be called by a batch processor
        this one needs no parameter
        """
        stdout = open(self.work_dir + 'stdout.txt', 'w')

        p = self.run_proc(['midway_video.sh',
                            self.work_dir,
                            self.bin_dir,
                            str(sigma),
                            str(dithering)], stdout=stdout)

        self.wait_proc(p, timeout)

        stdout.close()

        return

    @cherrypy.expose
    @init_app
    def result(self):
        """
        display the algo results
        """
        return self.tmpl_out("result.html")
